* Fiscal Year Calculations
*
* To include these functions in your FoxPro program:
*    
*    set procedure to fiscal.prg additive


* D = a date
* Returns a number from 1 to 52
function FiscalDateToWeek(m.d)
	* The week() function gets us what we want if the accounting year 
	* started with January.  Unfortunately, it does not do that.
	local m.FirstDay
	
	m.FirstDay = FiscalFirstDayInYear(m.d)
	
	* Count full weeks forward
	return 1 + int((m.d - m.FirstDay) / 7)
endfunc


* D = a date
* Returns a number from 1 to 12
function FiscalDateToMonth(m.d)
	local m.Week
	
	if empty(m.d)
		return 0
	endif

	m.Week = FiscalDateToWeek(m.d)
	return FiscalWeekToMonth(m.Week)
endfunc


* Year = fiscal year
* Month = fiscal month (jan=12, feb=1)
* Returns the date of the first day in the month (always is a Sunday)
function FiscalFirstDayOfMonth(m.Year, m.Month)
    local m.d, m.i
    m.d = FiscalFirstDayInYear(date(m.Year, m.Month, 1))
    if (year(m.d) != m.Year) then
    	m.d = FiscalFirstDayInYear(date(m.Year, m.Month + 6, 1))
    endif

    m.d = m.d + (FiscalWeeksBeforeMonth(m.Month) * 7)
    
    return m.d
endfunc


* Month = fiscal month (1 = feb, 12 = jan)
* Returns the number of weeks in a specific fiscal month
* Needs the year since January could have 5 weeks
function FiscalWeeksInMonth(m.Year, m.Month)
   if m.Month == 2 or m.Month == 5 or m.Month == 8 or m.Month == 11 then
      return 5
   endif
   if m.Month != 12
      return 4
   endif
   if mod(m.Year - 12, 28) == 0 then
      return 5
   endif
   return 4
endfunc


* Year = fiscal year
* Returns the number of weeks in the specified fiscal year
* Every 28 years, January has 5 weeks.
function FiscalWeeksInYear(m.Year)
	if mod(m.Year - 12, 28) == 0 then
		return 53
	endif
	return 52
endfunc
    

* w = the fiscal week number
* Returns the month number (February is month 1)
* Structured to work as fast as possible (faster than case statements)
function FiscalWeekToMonth(m.Week)
	if m.Week <= 26 then
		if m.Week <= 13 then
			if m.Week <= 4 then
				return 1
			endif
			if m.Week <= 9 then
				return 2
			endif
			return 3
		endif
		if m.Week <= 17 then
			return 4
		endif
		if m.Week <= 22 then
			return 5
		endif
		return 6
	endif
	if m.Week <= 39 then
		if m.Week <= 30 then
			return 7
		endif
		if m.Week <= 35 then
			return 8
		endif
		return 9
	endif
	if m.Week <= 43 then
		return 10
	endif
	if m.Week <= 48 then
		return 11
	endif
	return 12
endfunc


* month = the fiscal month number (1 = February)
* Returns the number of fiscal weeks before the fiscal month
* Again, structured to work as fast as possible
function FiscalWeeksBeforeMonth(m.Month)
	if m.Month <= 6 then
		if m.Month <= 3 then
			if m.Month == 1 then
				return 0
			endif
			if m.Month == 2 then
				return 4
			endif
			return 9
		endif
		if m.Month == 4 then
			return 13
		endif
		if m.Month == 5 then
			return 17
		endif
		return 22
	endif
	if m.Month <= 9 then
		if m.Month == 7 then
			return 26
		endif
		if m.Month == 8 then
			return 30
		endif
		return 35
	endif
	if m.Month == 10 then
		return 39
	endif
	if m.Month == 11 then
		return 43
	endif
	return 48
endfunc


* w = the fiscal week number
* February is month 1
* Returns the week number (1 through 5) of the fiscal month
* So, pass in week 6, get week 2 (second fiscal week of March)
* Pass in 10, get week 1 (first fiscal week of April)
function FiscalWeekToMonthWeek(m.Week)
	if m.Week <= 26 then
		if m.Week <= 13 then
			if m.Week <= 4 then
				return m.Week
			endif
			if m.Week <= 9 then
				return m.Week - 4
			endif
			return m.Week - 9
		endif
		if m.Week <= 17 then
			return m.Week - 13
		endif
		if m.Week <= 22 then
			return m.Week - 17
		endif
		return m.Week - 22
	endif
	if m.Week <= 39 then
		if m.Week <= 30 then
			return m.Week - 26
		endif
		if m.Week <= 35 then
			return m.Week - 30
		endif
		return m.Week - 35
	endif
	if m.Week <= 43 then
		return m.Week - 39
	endif
	if m.Week <= 48 then
		return m.Week - 43
	endif
	return m.Week - 48
endfunc


* D = a date
* Returns the fiscal year that the date falls in
* Feb is the first month in the year, so Jan is the last month of the 
* previous year
function FiscalDateToYear(m.d)
	if empty(m.d)
		return 0
	endif
	m.d = FiscalFirstDayInYear(m.d)
	return year(m.d)
endfunc


* D = a date
* Returns the first day (a date) in the fiscal year that D falls in
* The fiscal year startson the 5th calendar sunday
function FiscalFirstDayInYear(m.d)
	local m.FirstDay, m.the_year
	
	* Start at Jan 1st.  Step forward until we are on sunday.
	m.FirstDay = date(year(m.d), 1, 1)
	
	if dow(m.FirstDay, 1) > 1 then
		m.FirstDay = m.FirstDay + (8 - dow(m.FirstDay, 1))
	endif
	
	m.FirstDay = m.FirstDay + 28
	
	if m.FirstDay > m.d
		m.FirstDay = date(year(m.d) - 1, 1, 1)
		
		if dow(m.FirstDay, 1) > 1 then
			m.FirstDay = m.FirstDay + (8 - dow(m.FirstDay, 1))
		endif
		
		m.FirstDay = m.FirstDay + 28
	endif
	
	return m.FirstDay
endfunc
